<?php 
/*
 * Plugin Name: Metals API
 * Author: metals-api.com
 * Version: 2.4
 * Description: Official <a href="https://metals-api.com/">metals-api.com</a> plugin.
 * Author URL: https://metals-api.com
*/ 

class MetalsAPI { 

    function saveOption($name){
        if(isset($_POST[$name])){
            update_option($name, sanitize_text_field($_POST[$name]));
        }
    }

    function saveNumericOption($name){
        if(isset($_POST[$name])){
            update_option($name, sanitize_text_field(abs(intval($_POST[$name]))));
        }
    }

    function saveHTMLOption($name){ 
        if(isset($_POST[$name])){
            update_option($name, wp_kses_post($_POST[$name]));
        }
    }

    function render_settings(){ 
        $saved = 0;
        if(isset($_POST['save'])){
            if (!isset($_POST['thenonce']) || !wp_verify_nonce($_POST['thenonce'])) {
                wp_die("ERROR: Request not valid");
            }
            $this->saveOption('metalapi_apikey'); 
            $this->saveHTMLOption('metalapi_500_msg');
            $this->saveHTMLOption('metalapi_200_msg');
            $this->saveHTMLOption('metalapi_000_msg');
            $saved = 1;
        }

        $metalapi_apikey = get_option('metalapi_apikey'); 
        $metalapi_200_msg = stripslashes(get_option('metalapi_200_msg', '{{base}} {{price}} {{unit}}'));
        $metalapi_500_msg = stripslashes(get_option('metalapi_500_msg', 'Error: {{error}}'));
        $metalapi_000_msg = stripslashes(get_option('metalapi_000_msg', 'Error: no answer')); 
        $metalapi_error = get_option('metalapi_error'); 

        require plugin_dir_path(__FILE__). 'templates/settings.php';
    }

    function render_info(){
        require plugin_dir_path(__FILE__). 'templates/info.php';
    }

    function addSettings(){
        add_menu_page('Metals API Settings', 'Metals API', 'administrator', 'metalsapi', [$this, 'render_settings']);
        add_submenu_page('metalsapi', 'Metals API Settings', 'Settings', 'administrator', 'metalsapisettings', [$this, 'render_settings']);
        add_submenu_page('metalsapi', 'Information', 'Shortcodes', 'administrator', 'aboutshortcodemetals', [$this, 'render_info']);
        remove_submenu_page("metalsapi", "metalsapi");
    }

    function render($attributes){
        $code = get_option('metalapi_000_msg', 'Error: no answer');
        $name_to_code = [
            "ruthenium" => "RUTH",
            "silver" => "XAG",
            "gold" => "XAU",
            "palladium" => "XPD",
            "platinum" => "XPT",
            "rhodium" => "XRH"
        ];

        $key = get_option('metalapi_apikey');
        $api = isset($attributes["date"]) && !empty(trim($attributes["date"])) ? date("Y-m-d", strtotime($attributes["date"])) : 'latest';
        $base = !empty(trim($attributes["base"])) ? trim($attributes["base"]) : 'USD';
        $symbol = trim($attributes["symbol"], '“”');

        $uri = "https://metals-api.com/api/$api?access_key=$key&base=".$base."&symbols=".strtoupper($symbol);
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $uri);
        curl_setopt($curl, CURLOPT_HEADER, 0);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        $response = curl_exec($curl); 
        curl_close($curl);

        if (isset($_GET['depurando'])) {
            echo 'URL de petición: '.print_r($uri, true).'<br/>'; 
            echo 'Respuesta: '.print_r($response, true).'<br/>';           
        }

        if (!empty(trim($response))) {
            $json = json_decode($response, true);
            $symbol_user = strtoupper($attributes["symbol"]);
            
            if (isset($json['success']) && $json['success'] == true) { 
                $code = get_option('metalapi_200_msg', '{{base}} {{price}} {{unit}}');
                if (!empty(trim($code))) {
                    if (isset($json["timestamp"])) {
                        $code = str_replace("{{timestamp}}", date($attributes['date-format'], $json["timestamp"]), $code);
                    }
                    
                    if (isset($json["date"])) {
                        $datetime = isset($attributes["date-format"]) && !empty(trim($attributes["date-format"])) ? date($attributes["date-format"], strtotime($json["date"])) : $json["date"];
                        $code = str_replace("{{date}}", $datetime, $code);
                    }

                    $currency_symbols = [
                        'USD' => '$',
                        'EUR' => '€',
                        'GBP' => '£',
                    ];
                    $currency_symbol = isset($currency_symbols[$base]) ? $currency_symbols[$base] : '$';
                    $code = str_replace("{{base}}", $currency_symbol, $code);
                    
                    if (isset($json["rates"])) {
                        $priced = $json["rates"][$symbol_user] ?? 0;
                        if (!empty($attributes["unit"])) {
                            switch ($attributes["unit"]) {
                                case "kg":
                                    $priced = $priced * 32.151;
                                    break;
                                case "gram":
                                    $priced = $priced * 31.103;
                                    break;
                                case "ounce":
                                    $priced = $priced;
                                    break;
                                default:
                                    $priced = $priced;
                                    break;
                            }
                        }
                        
                        if (isset($attributes["price-round"])) {
                            $round_price = intval(trim($attributes["price-round"], '“”'));
                            $priced = round($priced, $round_price);
                        }
                        
                        $separator = isset($attributes["separator"]) ? trim($attributes["separator"]) : '.';
                        $priced = number_format($priced, isset($attributes["price-round"]) ? intval(trim($attributes["price-round"], '“”')) : 2, $separator, ',');
                        
                        return $priced;
                        
                        $code = str_replace('{{price}}', $priced, $code);
                        $code = str_replace("{{symbol}}", $attributes["symbol"], $code);

                    }
                    
                    if (isset($attributes["unit"])) {
                        switch ($attributes["unit"]) {
                            case "kg":
                                $code = str_replace("{{unit}}", 'Kg', $code);
                                break;
                            case "gram":
                                $code = str_replace("{{unit}}", 'Gram', $code);
                                break;
                            case "ounce":
                                $code = str_replace("{{unit}}", 'Ounce', $code);
                                break;
                            default:
                                $code = str_replace("{{unit}}", '', $code);
                                break;
                        }
                    }
                    
                    update_option('metalapi_error', 'Working'); 
                } else {
                    $code = get_option('metalapi_500_msg', 'Error: {{error}}');
                    update_option('metalapi_error', 'stopped');
                    if (isset($json["error"]["info"])) {
                        update_option('metalapi_error', "Error: [".$json["error"]["info"]."]");
                        $code = str_replace("{{error}}", $json["error"]["info"], $code);
                    }
                }
            }
        }
        return $code;
    }
}

$metalsAPI = new MetalsAPI();
add_action('admin_menu', [$metalsAPI, 'addSettings']);
add_shortcode('metals_api_price', [$metalsAPI, 'render']);
?>